// project.cpp

#include "stdafx.h"
#include <vector>
#include <set>
#include "ICsProject.h"
#include "ICsFile.h"
#include "IAssembly.h"
#include "ICsParserInterface.h"
#include "PerfomanceStatistics.h"

#include "CommandLineOptions.h"
#include "CsFormatter.h"

void format_file ( ICsFile * file );

//--------------------------------------------------------------

void process_project
(
	std::vector<ICsFile*> & files,
	std::vector<IAssembly*> & assemblies,
	std::set<std::string> & macros,
	PerfomanceStatistics & perfomaceStatistics
)
{
	ICsProject * project = ICsProject::create();
	project->setBuildEntityModel ( false );

	project->attachFiles ( (int)files.size(), & files[0] );
	project->attachAssemblies ( (int)assemblies.size(), & assemblies[0] );
	{
		std::set<std::string>::iterator it;
		for ( it = macros.begin(); it != macros.end(); ++it )
		{
			project->addMacro ( it->c_str() );
		}
	}

	project->parse();

	std::vector<ICsFile*>::iterator & fit = files.begin();
	std::vector<ICsFile*>::iterator & fit_end = files.end();
	for ( ; fit != fit_end; ++fit )
	{
		format_file ( *fit );
	}

	delete project;
}

//--------------------------------------------------------------

void format_file ( ICsFile * file )
{
	//#if COLLECT_FORMATTER_PERFOMANCE
	//	PerfomanceCounter formatter_time;
	//	formatter_time.start();
	//#endif // COLLECT_FORMATTER_PERFOMANCE

	CsFormatter formatter;
	CsCompilationUnit * unit = file->getCompilationUnit();
	if ( !unit )
	{
		ASSERT(0);
		return;
	}
	formatter.generate ( unit );

	bool b = true;
	std::string output_file;
	std::string fullPath = file->getPath();

	if ( b )
	{
		b = EvalPath (
			fullPath.c_str(),
			getCommandLineOptions().input_base_path.c_str(),
			getCommandLineOptions().output_base_path.c_str(),
			output_file );
	}

	std::string directory;
	b = GetDirectoryFromPath ( output_file.c_str(), directory );
	FileManager::createDirectoryRecursive ( directory.c_str() );

	if ( b )
	{
		PFile file ( new File ( output_file.c_str() ) );
		file->open ( File::ac_create );
		CharDeque::part_iterator it = formatter.generated_text_.part_begin();
		CharDeque::part_iterator it_end = formatter.generated_text_.part_end();
		for ( ; it != it_end; ++it )
		{
			CharDeque::part part = * it;
			file->write ( (byte*)part.begin(), part.end() - part.begin() );
		}
		unsigned file_len = 0;
		file->getLength ( file_len );
		file->close();
	}

	//#if COLLECT_FORMATTER_PERFOMANCE
	//	formatter_time.end();
	//	perfomanceStatistics_.formatter_time += formatter_time.getData();
	//	perfomanceStatistics_.formatter_generated_bytes_ += formatter.buffer_.eval_size();;
	//#endif // COLLECT_FORMATTER_PERFOMANCE
}

//--------------------------------------------------------------
